@@ -915,169 +915,3 @@ int havalFinal (havalContext *hcp, byte *digest)
 } /* havalFinal */
 
 
-#ifdef SELF_TESTING
-
-#include <stdio.h>
-
-
-static void printDigest (const char *tag, const byte *digest, size_t length)
-{
-	size_t i;
-
-	length >>= 3; /* convert bit length to byte length */
-	printf ("%s = ", tag);
-	for (i = 0; i < length; i++) {
-		printf ("%02X", digest [i]);
-	}
-	printf ("\n");
-} /* printDigest */
-
-
-int main (int argc, char *argv[])
-{
-	havalContext hc;
-	byte digest [32];
-
-	printf ("HAVAL test -- compiled on " __DATE__ " " __TIME__".\n\n");
-
-	switch (argc == 2 ? atoi (argv[1]) : 0) {
-	case 3:
-		havalInit   (&hc, 3, 128);
-		havalUpdate (&hc, (byte*) "", 0);
-		havalFinal  (&hc, digest);
-		printf      ("HAVAL(3,128,\"\")\n");
-		printDigest ("evaluated", digest, 128);
-		printf      ("expected  = C68F39913F901F3DDF44C707357A7D70\n\n");
-
-		havalInit   (&hc, 3, 160);
-		havalUpdate (&hc, (byte*)"a", 1);
-		havalFinal  (&hc, digest);
-		printf      ("HAVAL(3,160,\"a\")\n");
-		printDigest ("evaluated", digest, 160);
-		printf      ("expected  = 4DA08F514A7275DBC4CECE4A347385983983A830\n\n");
-
-		havalInit   (&hc, 3, 192);
-		havalUpdate (&hc, (byte*)"HAVAL", strlen ("HAVAL"));
-		havalFinal  (&hc, digest);
-		printf      ("HAVAL(3,192,\"HAVAL\")\n");
-		printDigest ("evaluated", digest, 192);
-		printf      ("expected  = 8DA26DDAB4317B392B22B638998FE65B0FBE4610D345CF89\n\n");
-
-		havalInit   (&hc, 3, 224);
-		havalUpdate (&hc, (byte*)"0123456789", strlen ("0123456789"));
-		havalFinal  (&hc, digest);
-		printf      ("HAVAL(3,224,\"0123456789\")\n");
-		printDigest ("evaluated", digest, 224);
-		printf      ("expected  = EE345C97A58190BF0F38BF7CE890231AA5FCF9862BF8E7BEBBF76789\n\n");
-
-		havalInit   (&hc, 3, 256);
-		havalUpdate (&hc, (byte*)"abcdefghijklmnopqrstuvwxyz", strlen ("abcdefghijklmnopqrstuvwxyz"));
-		havalFinal  (&hc, digest);
-		printf      ("HAVAL(3,256,\"abcdefghijklmnopqrstuvwxyz\")\n");
-		printDigest ("evaluated", digest, 256);
-		printf      ("expected  = 72FAD4BDE1DA8C8332FB60561A780E7F504F21547B98686824FC33FC796AFA76\n\n");
-
-		havalInit   (&hc, 3, 256);
-		havalUpdate (&hc, (byte*)"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789", strlen ("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789"));
-		havalFinal  (&hc, digest);
-		printf      ("HAVAL(3,256,\"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789\")\n");
-		printDigest ("evaluated", digest, 256);
-		printf      ("expected  = 899397D96489281E9E76D5E65ABAB751F312E06C06C07C9C1D42ABD31BB6A404\n\n");
-		break;
-
-	case 4:
-		havalInit   (&hc, 4, 128);
-		havalUpdate (&hc, (byte*)"", 0);
-		havalFinal  (&hc, digest);
-		printf      ("HAVAL(4,128,\"\")\n");
-		printDigest ("evaluated", digest, 128);
-		printf      ("expected  = EE6BBF4D6A46A679B3A856C88538BB98\n\n");
-
-		havalInit   (&hc, 4, 160);
-		havalUpdate (&hc, (byte*)"a", 1);
-		havalFinal  (&hc, digest);
-		printf      ("HAVAL(4,160,\"a\")\n");
-		printDigest ("evaluated", digest, 160);
-		printf      ("expected  = E0A5BE29627332034D4DD8A910A1A0E6FE04084D\n\n");
-
-		havalInit   (&hc, 4, 192);
-		havalUpdate (&hc, (byte*)"HAVAL", strlen ("HAVAL"));
-		havalFinal  (&hc, digest);
-		printf      ("HAVAL(4,192,\"HAVAL\")\n");
-		printDigest ("evaluated", digest, 192);
-		printf      ("expected  = 0C1396D7772689C46773F3DAACA4EFA982ADBFB2F1467EEA\n\n");
-
-		havalInit   (&hc, 4, 224);
-		havalUpdate (&hc, (byte*)"0123456789", strlen ("0123456789"));
-		havalFinal  (&hc, digest);
-		printf      ("HAVAL(4,224,\"0123456789\")\n");
-		printDigest ("evaluated", digest, 224);
-		printf      ("expected  = BEBD7816F09BAEECF8903B1B9BC672D9FA428E462BA699F814841529\n\n");
-
-		havalInit   (&hc, 4, 256);
-		havalUpdate (&hc, (byte*)"abcdefghijklmnopqrstuvwxyz", strlen ("abcdefghijklmnopqrstuvwxyz"));
-		havalFinal  (&hc, digest);
-		printf      ("HAVAL(4,256,\"abcdefghijklmnopqrstuvwxyz\")\n");
-		printDigest ("evaluated", digest, 256);
-		printf      ("expected  = 124F6EB645DC407637F8F719CC31250089C89903BF1DB8FAC21EA4614DF4E99A\n\n");
-
-		havalInit   (&hc, 4, 256);
-		havalUpdate (&hc, (byte*)"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789", strlen ("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789"));
-		havalFinal  (&hc, digest);
-		printf      ("HAVAL(4,256,\"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789\")\n");
-		printDigest ("evaluated", digest, 256);
-		printf      ("expected  = 46A3A1DFE867EDE652425CCD7FE8006537EAD26372251686BEA286DA152DC35A\n\n");
-		break;
-
-	case 5:
-		havalInit   (&hc, 5, 128);
-		havalUpdate (&hc, (byte*)"", 0);
-		havalFinal  (&hc, digest);
-		printf      ("HAVAL(5,128,\"\")\n");
-		printDigest ("evaluated", digest, 128);
-		printf      ("expected  = 184B8482A0C050DCA54B59C7F05BF5DD\n\n");
-
-		havalInit   (&hc, 5, 160);
-		havalUpdate (&hc, (byte*)"a", 1);
-		havalFinal  (&hc, digest);
-		printf      ("HAVAL(5,160,\"a\")\n");
-		printDigest ("evaluated", digest, 160);
-		printf      ("expected  = F5147DF7ABC5E3C81B031268927C2B5761B5A2B5\n\n");
-
-		havalInit   (&hc, 5, 192);
-		havalUpdate (&hc, (byte*)"HAVAL", strlen ("HAVAL"));
-		havalFinal  (&hc, digest);
-		printf      ("HAVAL(5,192,\"HAVAL\")\n");
-		printDigest ("evaluated", digest, 192);
-		printf      ("expected  = 794A896D1780B76E2767CC4011BAD8885D5CE6BD835A71B8\n\n");
-
-		havalInit   (&hc, 5, 224);
-		havalUpdate (&hc, (byte*)"0123456789", strlen ("0123456789"));
-		havalFinal  (&hc, digest);
-		printf      ("HAVAL(5,224,\"0123456789\")\n");
-		printDigest ("evaluated", digest, 224);
-		printf      ("expected  = 59836D19269135BC815F37B2AEB15F894B5435F2C698D57716760F2B\n\n");
-
-		havalInit   (&hc, 5, 256);
-		havalUpdate (&hc, (byte*)"abcdefghijklmnopqrstuvwxyz", strlen ("abcdefghijklmnopqrstuvwxyz"));
-		havalFinal  (&hc, digest);
-		printf      ("HAVAL(5,256,\"abcdefghijklmnopqrstuvwxyz\")\n");
-		printDigest ("evaluated", digest, 256);
-		printf      ("expected  = C9C7D8AFA159FD9E965CB83FF5EE6F58AEDA352C0EFF005548153A61551C38EE\n\n");
-
-		havalInit   (&hc, 5, 256);
-		havalUpdate (&hc, (byte*)"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789", strlen ("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789"));
-		havalFinal  (&hc, digest);
-		printf      ("HAVAL(5,256,\"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789\")\n");
-		printDigest ("evaluated", digest, 256);
-		printf      ("expected  = B45CB6E62F2B1320E4F8F1B0B273D45ADD47C321FD23999DCF403AC37636D963\n\n");
-		break;
-
-	default:
-		printf ("usage: haval <passes>\n");
-		break;
-	}
-	return 0;
-} /* main */
-
-#endif /* ?SELF_TESTING */
