@@ -19,7 +19,7 @@
  */
 
 
-/* $Id: mhash.c,v 1.13 2000/12/15 12:42:33 nmav Exp $ */
+/* $Id: mhash.c,v 1.16 2001/01/21 19:27:29 nmav Exp $ */
 
 #include <stdlib.h>
 
@@ -33,6 +33,7 @@
 #include "mhash_crc32.h"
 #include "mhash_haval.h"
 #include "mhash_md5.h"
+#include "mhash_md4.h"
 #include "mhash_sha1.h"
 #include "mhash_tiger.h"
 #include "mhash_ripemd.h"
@@ -53,6 +54,7 @@ struct mhash_hash_entry {
 static mhash_hash_entry algorithms[] = {
 	MHASH_ENTRY(MHASH_CRC32, 4, 0),
 	MHASH_ENTRY(MHASH_MD5, 16, 64),
+	MHASH_ENTRY(MHASH_MD4, 16, 64),
 	MHASH_ENTRY(MHASH_SHA1, 20, 64),
 	MHASH_ENTRY(MHASH_HAVAL256, 32, 128),
 	MHASH_ENTRY(MHASH_HAVAL128, 16, 128),
@@ -60,7 +62,9 @@ static mhash_hash_entry algorithms[] = {
 	MHASH_ENTRY(MHASH_HAVAL192, 24, 128),
 	MHASH_ENTRY(MHASH_HAVAL224, 28, 128),
 	MHASH_ENTRY(MHASH_RIPEMD160, 20, 64),
-	MHASH_ENTRY(MHASH_TIGER, 192 >> 3, 64),
+	MHASH_ENTRY(MHASH_TIGER, 24, 64),
+	MHASH_ENTRY(MHASH_TIGER128, 16, 64),
+	MHASH_ENTRY(MHASH_TIGER160, 20, 64),
 	MHASH_ENTRY(MHASH_GOST, 32, 0),
 	MHASH_ENTRY(MHASH_CRC32B, 4, 0),
 	{0}
@@ -95,7 +99,7 @@ WIN32DLL_DEFINE size_t mhash_get_block_size(hashid type)
 char *mystrdup(char *str)
 {
 	char *ret;
-	ret = malloc(strlen(str) + 1);
+	if ( (ret = malloc(strlen(str) + 1)) == NULL) return NULL;
 	strcpy(ret, str);
 
 	return ret;
@@ -119,16 +123,16 @@ WIN32DLL_DEFINE char *mhash_get_hash_name(hashid type)
 MHASH mhash_cp(MHASH from) {
 MHASH ret;
 
-	ret = malloc(sizeof(MHASH_INSTANCE));
+	if ( (ret = malloc(sizeof(MHASH_INSTANCE))) == NULL) return MHASH_FAILED;
 	memcpy(ret, from, sizeof(MHASH_INSTANCE));
 	
 	/* copy the internal state also */
-	ret->state=malloc(ret->state_size);
+	if ( (ret->state=malloc(ret->state_size)) == NULL) return MHASH_FAILED;
 	memcpy( ret->state, from->state, ret->state_size);
 	
 	/* copy the key in case of hmac*/
 	if (ret->hmac_key_size!=0) {
-		ret->hmac_key=malloc(ret->hmac_key_size);
+		if ((ret->hmac_key=malloc(ret->hmac_key_size)) == NULL) return MHASH_FAILED;
 		memcpy(ret->hmac_key, from->hmac_key, ret->hmac_key_size);
 	}
 	return ret;
@@ -139,7 +143,7 @@ MHASH mhash_init_int(const hashid type)
 	MHASH ret;
 	int i;
 
-	ret = malloc(sizeof(MHASH_INSTANCE));
+	if ( (ret = malloc(sizeof(MHASH_INSTANCE))) == NULL) return MHASH_FAILED;
 	ret->algorithm_given = type;
 	ret->hmac_key = NULL;
 	ret->state = NULL;
@@ -149,56 +153,64 @@ MHASH mhash_init_int(const hashid type)
 	case MHASH_CRC32:
 	case MHASH_CRC32B:
 		ret->state_size = sizeof(word32);
-		ret->state = malloc(ret->state_size);
+		if ( (ret->state = malloc(ret->state_size)) == NULL) return MHASH_FAILED;
 		clear_crc32((void *) ret->state);
 		break;
 	case MHASH_MD5:
 		ret->state_size = sizeof(MD5_CTX);
-		ret->state = malloc(ret->state_size);
+		if ( (ret->state = malloc(ret->state_size)) == NULL) return MHASH_FAILED;
 		MD5Init((void *) ret->state);
 		break;
+	case MHASH_MD4:
+		ret->state_size = sizeof(MD4_CTX);
+		if ( (ret->state = malloc(ret->state_size)) == NULL) return MHASH_FAILED;
+		MD4Init((void *) ret->state);
+		break;
 	case MHASH_SHA1:
 		ret->state_size = sizeof(SHA_CTX);
-		ret->state = malloc(ret->state_size);
+		if ( (ret->state = malloc(ret->state_size)) == NULL) return MHASH_FAILED;
 		sha_init((void *) ret->state);
 		break;
 	case MHASH_HAVAL256:
 		ret->state_size = sizeof(havalContext);
-		ret->state = malloc(ret->state_size);
+		if ( (ret->state = malloc(ret->state_size)) == NULL) return MHASH_FAILED;
 		havalInit((void *) ret->state, 3, 256);
 		break;
 	case MHASH_HAVAL224:
 		ret->state_size = sizeof(havalContext);
-		ret->state = malloc(ret->state_size);
+		if ( (ret->state = malloc(ret->state_size)) == NULL) return MHASH_FAILED;
 		havalInit((void *) ret->state, 3, 224);
 		break;
 	case MHASH_HAVAL192:
 		ret->state_size = sizeof(havalContext);
-		ret->state = malloc(ret->state_size);
+		if ( (ret->state = malloc(ret->state_size)) == NULL) return MHASH_FAILED;
 		havalInit((void *) ret->state, 3, 192);
 		break;
 	case MHASH_HAVAL160:
 		ret->state_size = sizeof(havalContext);
-		ret->state = malloc(ret->state_size);
+		if ( (ret->state = malloc(ret->state_size)) == NULL) return MHASH_FAILED;
 		havalInit((void *) ret->state, 3, 160);
 		break;
 	case MHASH_HAVAL128:
 		ret->state_size = sizeof(havalContext);
-		ret->state = malloc(ret->state_size);
+		if ( (ret->state = malloc(ret->state_size)) == NULL) return MHASH_FAILED;
 		havalInit((void *) ret->state, 3, 128);
 		break;
 	case MHASH_RIPEMD160:
 		ret->state_size = sizeof(RIPEMD_CTX);
-		ret->state = malloc(ret->state_size);
+		if ( (ret->state = malloc(ret->state_size)) == NULL) return MHASH_FAILED;
 		ripemd_init((void *) ret->state);
 		break;
 	case MHASH_TIGER:
-		ret->state_size = 3 * sizeof(word64);
-		ret->state = malloc(ret->state_size);
+	case MHASH_TIGER128:
+	case MHASH_TIGER160:
+		ret->state_size = sizeof(TIGER_CTX);
+		if ( (ret->state = malloc(ret->state_size)) == NULL) return MHASH_FAILED;
+		tiger_init((void*) ret->state);
 		break;
 	case MHASH_GOST:
 		ret->state_size = sizeof(GostHashCtx);
-		ret->state = malloc(ret->state_size);
+		if ( (ret->state = malloc(ret->state_size)) == NULL) return MHASH_FAILED;
 		gosthash_reset((void *) ret->state);
 		break;
 	default:
@@ -244,6 +256,9 @@ int mhash(MHASH thread, const void *plaintext, size_t size)
 	case MHASH_MD5:
 		MD5Update((void *) thread->state, plaintext, size);
 		break;
+	case MHASH_MD4:
+		MD4Update((void *) thread->state, plaintext, size);
+		break;
 	case MHASH_SHA1:
 		sha_update((void *) thread->state, (void *) plaintext,
 			   size);
@@ -260,7 +275,9 @@ int mhash(MHASH thread, const void *plaintext, size_t size)
 			      size);
 		break;
 	case MHASH_TIGER:
-		tiger(plaintext, size, (void *) thread->state);
+	case MHASH_TIGER160:
+	case MHASH_TIGER128:
+		tiger_update((void*)thread->state, (void*)plaintext, size);
 		break;
 	case MHASH_GOST:
 		gosthash_update((void *) thread->state, plaintext, size);
@@ -280,18 +297,29 @@ WIN32DLL_DEFINE
 	case MHASH_CRC32:
 	case MHASH_CRC32B:
 		rtmp = hash_malloc(sizeof(word32));
+		if (rtmp==NULL) return NULL;
 		get_crc32(rtmp, (void *) thread->state);
 		break;
 	case MHASH_MD5:
 		digest =
 		    hash_malloc(mhash_get_block_size
 				(thread->algorithm_given));
+		if (digest == NULL) return NULL;
 		MD5Final(digest, (void *) thread->state);
 		rtmp = digest;
 		break;
+	case MHASH_MD4:
+		digest =
+		    hash_malloc(mhash_get_block_size
+				(thread->algorithm_given));
+		if (digest == NULL) return NULL;
+		MD4Final(digest, (void *) thread->state);
+		rtmp = digest;
+		break;
 	case MHASH_SHA1:
 		sha_final((void *) thread->state);
 		digest = hash_malloc(SHA_DIGESTSIZE);
+		if (digest == NULL) return NULL;
 		sha_digest((void *) thread->state, digest);
 		rtmp = digest;
 		break;
@@ -303,23 +331,41 @@ WIN32DLL_DEFINE
 		digest =
 		    hash_malloc(mhash_get_block_size
 				(thread->algorithm_given));
+		if (digest == NULL) return NULL;
 		havalFinal((void *) thread->state, digest);
 		rtmp = digest;
 		break;
 	case MHASH_RIPEMD160:
 		ripemd_final((void *) thread->state);
 		digest = hash_malloc(RIPEMD_DIGESTSIZE);
+		if (digest == NULL) return NULL;
 		ripemd_digest((void *) thread->state, digest);
 		rtmp = digest;
 		break;
 	case MHASH_TIGER:
-		digest = hash_malloc(192 >> 3);
-		memcpy(digest, (void *) thread->state, 192 >> 3);
-		mhash_32bit_conversion(digest, 192 >> 5);
+		tiger_final((void*) thread->state);
+		digest = hash_malloc(TIGER_DIGESTSIZE);
+		if (digest == NULL) return NULL;
+		tiger_digest((void *) thread->state, digest);
+		rtmp = digest;
+		break;
+	case MHASH_TIGER128:
+		tiger_final((void*) thread->state);
+		digest = hash_malloc(TIGER128_DIGESTSIZE);
+		if (digest == NULL) return NULL;
+		tiger128_digest((void *) thread->state, digest);
+		rtmp = digest;
+		break;
+	case MHASH_TIGER160:
+		tiger_final((void*) thread->state);
+		digest = hash_malloc(TIGER160_DIGESTSIZE);
+		if (digest == NULL) return NULL;
+		tiger160_digest((void *) thread->state, digest);
 		rtmp = digest;
 		break;
 	case MHASH_GOST:
 		digest = hash_malloc(32);
+		if (digest == NULL) return NULL;
 		gosthash_final((void *) thread->state, digest);
 		rtmp = digest;
 		break;
@@ -369,6 +415,7 @@ WIN32DLL_DEFINE
 	int i;
 
 	opad = malloc(thread->hmac_block);
+	if (opad == NULL) return NULL;
 
 	for (i = 0; i < thread->hmac_key_size; i++) {
 		opad[i] = (0x5C) ^ thread->hmac_key[i];
@@ -384,6 +431,7 @@ WIN32DLL_DEFINE
 	case MHASH_CRC32:
 	case MHASH_CRC32B:
 		digest = hash_malloc(sizeof(word32));
+		if (digest == NULL) return NULL;
 		get_crc32(digest, (void *) thread->state);
 		return_val = digest;
 		break;
@@ -391,11 +439,21 @@ WIN32DLL_DEFINE
 		digest =
 		    hash_malloc(mhash_get_block_size
 				(thread->algorithm_given));
+		if (digest == NULL) return NULL;
 		MD5Final(digest, (void *) thread->state);
 		return_val = digest;
 		break;
+	case MHASH_MD4:
+		digest =
+		    hash_malloc(mhash_get_block_size
+				(thread->algorithm_given));
+		if (digest == NULL) return NULL;
+		MD4Final(digest, (void *) thread->state);
+		return_val = digest;
+		break;
 	case MHASH_SHA1:
 		digest = hash_malloc(SHA_DIGESTSIZE);
+		if (digest == NULL) return NULL;
 		sha_final((void *) thread->state);
 		sha_digest((void *) thread->state, digest);
 		return_val = digest;
@@ -408,22 +466,41 @@ WIN32DLL_DEFINE
 		digest =
 		    hash_malloc(mhash_get_block_size
 				(thread->algorithm_given));
+		if (digest == NULL) return NULL;
 		havalFinal((void *) thread->state, digest);
 		return_val = digest;
 		break;
 	case MHASH_RIPEMD160:
 		digest = hash_malloc(RIPEMD_DIGESTSIZE);
+		if (digest == NULL) return NULL;
 		ripemd_final((void *) thread->state);
 		ripemd_digest((void *) thread->state, digest);
 		return_val = digest;
 		break;
 	case MHASH_TIGER:
-		digest = hash_malloc(192 >> 3);
-		memcpy(digest, (void *) thread->state, 192 >> 3);
+		digest = hash_malloc(TIGER_DIGESTSIZE);
+		if (digest == NULL) return NULL;
+		tiger_final((void *) thread->state);
+		tiger_digest((void *) thread->state, digest);
+		return_val = digest;
+		break;
+	case MHASH_TIGER128:
+		digest = hash_malloc(TIGER128_DIGESTSIZE);
+		if (digest == NULL) return NULL;
+		tiger_final((void *) thread->state);
+		tiger128_digest((void *) thread->state, digest);
+		return_val = digest;
+		break;
+	case MHASH_TIGER160:
+		digest = hash_malloc(TIGER160_DIGESTSIZE);
+		if (digest == NULL) return NULL;
+		tiger_final((void *) thread->state);
+		tiger160_digest((void *) thread->state, digest);
 		return_val = digest;
 		break;
 	case MHASH_GOST:
 		digest = hash_malloc(32);
+		if (digest == NULL) return NULL;
 		gosthash_final((void *) thread->state, digest);
 		return_val = digest;
 		break;
@@ -472,6 +549,7 @@ WIN32DLL_DEFINE
 		ret->hmac_block = block;
 
 		ipad = malloc(ret->hmac_block);
+		if (ipad == NULL) return MHASH_FAILED;
 
 		if (keysize > ret->hmac_block) {
 			tmptd = mhash_init(type);
