@@ -19,7 +19,10 @@ enum hashid {
 	MHASH_HAVAL224=10,
 	MHASH_HAVAL192,
 	MHASH_HAVAL160,
-	MHASH_HAVAL128
+	MHASH_HAVAL128,
+	MHASH_TIGER128,
+	MHASH_TIGER160,
+	MHASH_MD4
 };
 
 enum keygenid {
